#!/usr/bin/python3
"""Setup
"""
from setuptools import find_packages
from distutils.core import setup

version = "1.2.5"

with open('README.rst') as f:
    long_description = f.read()

setup(name='ofxstatement-czech',
      version=version,
      author="Matěj Cepl",
      author_email="mcepl@cepl.eu",
      url="https://gitlab.com/mcepl/ofxstatement-czech",
      description=("Czech banks plugin for ofxstatement"),
      long_description=long_description,
      license="GPLv3",
      keywords=["ofx", "ofxstatement"],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: GNU General Public License v3'],
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=["ofxstatement", "ofxstatement.plugins"],
      entry_points={
          'ofxstatement':
          ['gpc = ofxstatement.plugins.gpc:GPCPlugin',
           'maxibps = ofxstatement.plugins.maxibps:PSPlugin']
      },
      install_requires=['ofxstatement'],
      test_suite="src.ofxstatement.plugins.tests",
      include_package_data=True,
      zip_safe=True
      )



import csv

from ofxstatement import statement
from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.parser import StatementParser
from ofxstatement.statement import StatementLine

class IngBePlugin(Plugin):
    """ING Belgium Plugin
    """

    def get_parser(self, filename):
        f = open(filename, 'r', encoding=self.settings.get("charset", "ISO-8859-1"))
        parser = IngBeParser(f)
        return parser

class IngBeParser(CsvStatementParser):

    date_format = "%d/%m/%Y"
    mappings = {
        'check_no': 3,
        'date': 4,
        'payee': 2,
        'memo': 9,
        'amount': 6
    }
    
    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        stmt = super(IngBeParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        
        reader = csv.reader(self.fin)
        next(reader, None)
        return reader

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        
        # Remove non CSV cr*p and zero-value notifications
        if(line[5] and not (line[6]=="0" and line[7]=="0")):          
            transaction_id = line[3]
            date = line[4]
            date_value = line[5]
            account_to = line[2]
            currency = line[8]

            # fix amount - Well done ING, mixing the comma as decimal separator and as CSV delimiter. Way to go...
            line[6] = line[6]+"."+line[7]                       
            amount = line[6]
            
            # Pack info in description
            line[9] = line[9]+"-"+line[10]+"-"+line[11] 
            description = line[9]

            stmtline = super(IngBeParser, self).parse_record(line)
            stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'
        
            return stmtline



import csv
from datetime import datetime
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
#from ofxstatement.statement import Statement


class RaiffeisenCZPlugin(Plugin):
    """Raiffeisenbank, a.s. (Czech Republic) (CSV, cp1250)
    """

    def get_parser(self, filename):
        # .csvfile is a work-around and is used for exporting fees to a new CSV file
        RaiffeisenCZPlugin.csvfile = re.sub(".csv", "", filename) + "-fees.csv"

        # Open input file and set some defaults
        RaiffeisenCZPlugin.encoding = self.settings.get('charset', 'cp1250')
        f = open(filename, "r", encoding=RaiffeisenCZPlugin.encoding)
        parser = RaiffeisenCZParser(f)
        parser.statement.currency = self.settings.get('currency', 'CZK')
        parser.statement.bank_id = self.settings.get('bank', 'RZBCCZPP')
        parser.statement.account_id = self.settings.get('account', '')
        parser.statement.account_type = self.settings.get('account_type', 'CHECKING')
        parser.statement.trntype = "OTHER"
        return parser


class RaiffeisenCZParser(CsvStatementParser):
    # The columns are:
    #  0 Datum
    #  1 Čas
    #  2 Poznámka
    #  3 Název účtu
    #  4 Číslo účtu
    #  5 Datum odepsání
    #  6 Valuta
    #  7 Typ
    #  8 Kód transakce
    #  9 Variabilní symbol
    # 10 Konstantní symbol
    # 11 Specifický symbol
    # 12 Částka
    # 13 Poplatek
    # 14 Směna
    # 15 Zpráva

    mappings = {"date_user": 0,
                "date": 5,
                "memo": 2,
                "payee": 3,
                "amount": 12,
                "check_no": 9,
                "refnum": 8, }

    date_format = "%d.%m.%Y"

    def split_records(self):
        return csv.reader(self.fin, delimiter=';', quotechar='"')

    def parse_record(self, line):
        if self.cur_record == 1:
            # Create a heading line for the -fees.csv file
            with open(RaiffeisenCZPlugin.csvfile, "w", encoding=RaiffeisenCZPlugin.encoding) as output:
                writer = csv.writer(output, lineterminator='\n', delimiter=';', quotechar='"')
                writer.writerow(line)
                output.close()
            # And skip further processing by parser
            return None

        if line[12] == '':
            line[12] = "0"

        if line[13] == '':
            line[13] = "0"

        if line[14] == '':
            line[14] = "0"

        if line[15] == '':
            line[15] = "0"

        sl = super(RaiffeisenCZParser, self).parse_record(line)
        sl.date_user = datetime.strptime(sl.date_user, self.date_format)

        sl.id = statement.generate_transaction_id(sl)

        if line[7].startswith("Převod"):
            sl.trntype = "XFER"
        if line[7].startswith("Příchozí platba"):
            sl.trntype = "CREDIT"
        if line[7].startswith("Trvalý převod"):
            sl.trntype = "REPEATPMT"
        if line[7].startswith("Kladný úrok"):
            sl.trntype = "INT"
        if line[7].startswith("Záporný úrok"):
            sl.trntype = "INT"
        if line[7].startswith("Inkaso"):
            sl.trntype = "DIRECTDEBIT"
        if line[7].startswith("Srážka daně"):
            sl.trntype = "DEBIT"
        if line[7].startswith("Správa účtu"):
            sl.trntype = "FEE"
        if line[7].startswith("Jiný trans."):
            sl.trntype = "FEE"
        if line[7].startswith("Správa účtu"):
            sl.trntype = "FEE"
        if line[7].startswith("Poplatek"):
            sl.trntype = "FEE"
        if line[7].startswith("Směna"):
            sl.trntype = "FEE"
        if line[7].startswith("Zpráva"):
            sl.trntype = "FEE"

        # .payee becomes OFX.NAME which becomes "Description" in GnuCash
        # .memo  becomes OFX.MEMO which becomes "Notes"       in GnuCash
        # When payee is empty, GnuCash imports .memo to "Description" and keeps "Notes" empty
        if not (line[4] == '' or line[4] == ' '):
            sl.payee = sl.payee + "|ÚČ: " + line[4]
        if not (line[9] == '' or line[9] == ' '):
            sl.memo = sl.memo + "|VS: " + line[9]
        if not (line[10] == '' or line[10] == ' '):
            sl.memo = sl.memo + "|KS: " + line[10]
        if not (line[11] == '' or line[11] == ' '):
            sl.memo = sl.memo + "|SS: " + line[11]

        # Raiffeisen may show various fees on the same line  as the underlying transaction
        # For now, we simply create a new CSV file with the fee (and only the fee) moved to line[12].
        # This needs to be processed again manually:
        # $ ofxstatement convert -t raiffeisencz in-fees.csv out-fees.ofx
        for x in range(13, 16):
            # ToDo: re-use parse_float (how??)
            val1 = re.sub(",", ".", line[x])
            val1 = re.sub("[ a-zA-Z]", "", val1)

            # Additional fee to transaction: export the fee to a new CSV
            # ToDo: instead of exporting the above to CSV, try to add the exportline to
            #       the end of statement (from imported input.csv).
            if float(val1) != 0 and sl.amount != 0:
                exportline = line[:]
                exportline[12] = line[x]

                if x == 13:
                    exportline[7] = "Poplatek"
                if x == 14:
                    exportline[7] = "Směna"
                if x == 15:
                    exportline[7] = "Zpráva"

                for y in range(13, 16):
                    exportline[y] = ''

                with open(RaiffeisenCZPlugin.csvfile, "a", encoding=RaiffeisenCZPlugin.encoding) as output:
                    writer = csv.writer(output, lineterminator='\n', delimiter=';', quotechar='"')
                    writer.writerow(exportline)

            # Some type of fee is standalone, not related to transaction amount. Add it to amount field.only
            # Most probably, there should not exist two standalone fees at once (possibly "Zpráva"?)
            if float(val1) != 0 and sl.amount == 0:
                sl.amount = sl.amount + float(val1)

        if sl.amount == 0:
            return None

        return sl

    def parse_float(self, value):
        value = re.sub(",", ".", value)
        value = re.sub("[ a-zA-Z]", "", value)
        return super(RaiffeisenCZParser, self).parse_float(value)



# -*- encoding: utf-8 -*-
import csv
import decimal

from ofxstatement.plugin import Plugin, PluginNotRegistered
from .parser import NordeaCsvStatementParser, SIGNATURES

class NordeaDialect(csv.Dialect):
   "Definitions for reading information from Nordea account statements"

   delimiter = "\t"
   doublequote = False
   escapechar = None
   lineterminator = "\r\n"
   quotechar = '"'
   quoting = csv.QUOTE_MINIMAL
   skipinitialspace = None


class NordeaPlugin(Plugin):
    "Suomen Osuuspankki / Finnish Osuuspankki"

    def get_csv_signature(self, csvfile):
        csvfile.readline() # account # line
        csvfile.readline() # first empty
        csvfile.readline() # second empty
        return csvfile.readline().strip()

    def get_parser(self, fin):
        f = open(fin, "r", encoding="1252")
        signature = self.get_csv_signature(f)
        if signature in SIGNATURES:
            parser = NordeaCsvStatementParser(f)
            parser.statement.account_id = self.settings['account']
            parser.statement.currency = self.settings['currency']
            parser.statement.bank_id = self.settings.get('bank', 'Nordea')
            return parser

        # no plugin with matching signature was found
        f.close()
        raise Exception("No suitable Nordea parser found for this statement file.")



# -*- coding: utf-8 -*-

import locale
import itertools
import csv
import re

from datetime import datetime

from contextlib import contextmanager
from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, generate_transaction_id


def take(iterable, n):
    """Return first n items of the iterable as a list."""
    return list(itertools.islice(iterable, n))


def drop(iterable, n):
    """Drop first n items of the iterable and return result as a list."""
    return list(itertools.islice(iterable, n, None))


def head(iterable):
    """Return first element of the iterable."""
    return take(iterable, 1)[0]


@contextmanager
def scoped_setlocale(category, loc=None):
    """Scoped version of locale.setlocale()"""
    orig = locale.getlocale(category)
    try:
        yield locale.setlocale(category, loc)
    finally:
        locale.setlocale(category, orig)


def atof(string, loc=None):
    """Locale aware atof function for our parser."""
    with scoped_setlocale(locale.LC_NUMERIC, loc):
        return locale.atof(string)


#
# Yeah, maybe CsvStatementParser is the right way to do it but we have some custom logic here
# that doesn't fit it nicely.
#
class AlfaBankStatementParser(StatementParser):
    bank_id = 'AlfaBank'
    date_format = '%d.%m.%y'

    def __init__(self, fin, encoding=None, locale=None, brief=False):
        self.locale = locale
        self.encoding = encoding
        self.brief = brief

        with open(fin, 'r', encoding=self.encoding) as f:
            self.lines = f.readlines()

        self.validate()
        self.statement = self.parse_statement()

    @property
    def reader(self):
        return csv.reader(self.lines, delimiter=';')

    @property
    def header(self):
        return head(self.reader)

    @property
    def rows(self):
        return drop(self.reader, 1)

    def validate(self):
        """
        Validate to ensure csv has the same header we expect.
        """

        expected = [
            u"Тип счёта",
            u"Номер счета",
            u"Валюта",
            u"Дата операции",
            u"Референс проводки",
            u"Описание операции",
            u"Приход",
            u"Расход",
            u""
        ]
        actual = self.header
        if expected != actual:
            msg = "\n".join([
                "Header template doesn't match:",
                "expected: %s" % expected,
                "actual  : %s" % actual
            ])
            raise ValueError(msg)

    def parse_statement(self):
        """
        Parse statement object.
        """

        account_ids = set()
        for row in self.rows:
            account_id = row[1]
            account_ids.add(account_id)
        if len(account_ids) < 1:
            raise ValueError("No accounts found: %s" % account_ids)
        if len(account_ids) > 1:
            raise ValueError("More than one account found: %s" % account_ids)

        currency_ids = set()
        for row in self.rows:
            currency_id = row[2]
            currency_ids.add(currency_id)
        if len(currency_ids) < 1:
            raise ValueError("No currency found: %s" % currency_ids)
        if len(currency_ids) > 1:
            raise ValueError("More than one currency found: %s" % currency_ids)

        return Statement(
            bank_id=self.bank_id,
            account_id=head(account_ids),
            currency=head(currency_ids),
        )

    def split_records(self):
        for row in self.rows:
            yield row

    def parse_record(self, row):
        account_name, account_id, currency, date, refnum, description, debit, credit = take(row, 8)

        stmt_line = StatementLine()
        stmt_line.date = datetime.strptime(date, self.date_format)
        stmt_line.memo = description
        stmt_line.refnum = refnum

        debit = atof(debit, self.locale)
        credit = atof(credit, self.locale)

        assert (debit > 0) ^ (credit > 0)
        if debit > 0:
            stmt_line.amount = debit
        elif credit > 0:
            stmt_line.amount = -1 * credit
        else:
            stmt_line.amount = 0.0

        #
        # Looks like AlfaBank formats description for card transactions so it includes the actual purchase date
        # within e.g.
        #
        # 123456++++++7890    12312312\111\NOVOSIBIRSK\CHITAI GOROD             10.01.16 08.01.16       617.00  RUR
        #
        # we cleanup it (optionally) by leaving only the actual description part i.e.
        #
        # 12312312\111\NOVOSIBIRSK\CHITAI GOROD
        #
        if refnum.startswith("CRD"):
            m = re.match('([0-9+]+) +(.+) +([0-9]{2}\.[0-9]{2}\.[0-9]{2}) ([0-9]{2}\.[0-9]{2}\.[0-9]{2}) +([0-9]+\.[0-9]+) +([^ ]+)', stmt_line.memo)
            if m:
                card_id, card_memo, _, card_date, card_amount, card_currency = m.groups()
                if self.brief:
                    stmt_line.memo = card_memo
                stmt_line.date_user = datetime.strptime(card_date, self.date_format)

        stmt_line.id = generate_transaction_id(stmt_line)
        return stmt_line


def parse_bool(value):
    if value in ('True', 'true', '1'):
        return True
    if value in ('False', 'false', '0'):
        return False
    raise ValueError("Can't parse boolean value: %s" % value)


class AlfaBankPlugin(Plugin):
    def get_parser(self, fin):
        kwargs = {
            'locale': 'ru_RU',
            'encoding': 'cp1251',
        }
        if self.settings:
            if 'brief' in self.settings:
                kwargs['brief'] = parse_bool(self.settings.get('brief'))
            if 'encoding' in self.settings:
                kwargs['encoding'] = self.settings.get('encoding')
            if 'locale' in self.settings:
                kwargs['locale'] = self.settings.get('locale')
        return AlfaBankStatementParser(fin, **kwargs)



from ofxstatement.parser import CsvStatementParser
import csv
from io import StringIO

from . import TRANSACTION_TYPES

# The bank for some reason has varied the column labeling a lot.
# By collecting first lines of known CSV exports as signatures
# we catch any future changes to the CSV export format.

SIGNATURES = (
"Arvop\xe4iv\xe4\tM\xe4\xe4r\xe4\Tapahtuma\tTilinumero\tSaaja/Maksaja\tViite\tViesti",
"Kirjauspäivä	Arvopäivä	Maksupäivä	Määrä	Saaja/Maksaja	Tilinumero	BIC	Tapahtuma	Viite	Maksajan viite	Viesti	Kortinnumero	Kuitti",
)

COLUMNS = 12

class NordeaCsvStatementParser(CsvStatementParser):
    "parser for various variations with common field semantics"
    
    mappings = {
       "date":0, "amount":3, "payee":4, "acctto":5, "trntype":7, "refnum":8, "memo":9
    }

    date_format = "%d.%m.%Y"

    def split_records(self):
        return csv.reader(self.fin, delimiter='\t', quotechar='"')

    def parse_record(self, line):
        # The plugin has iterated over the prefix lines for us already
        #if self.cur_record <= 3:
        #    return None

        # Change decimalsign from , to .
        line[3] = line[3].replace(',', '.')

        # Set transaction type
        line[7] = TRANSACTION_TYPES[line[7].upper()]

        # the CSV leaves off empty data at the end of record
        # so we fill in some blanks
        missing = COLUMNS - len(line)
        line.extend(missing * [""])

        # fill statement line according to mappings
        sl = super(NordeaCsvStatementParser, self).parse_record(line)
        return sl




import csv
from datetime import datetime
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement


class AirBankCZPlugin(Plugin):
    """Air Bank a.s. (Czech Republic) (CSV, UTF-8)
    NB: if there are any transaction related fees (column 06), a new CSV
        file is created and it has to be processed again:
        $ ofxstatement convert -t airbankcz in-fees.csv out-fees.ofx
    """

    def get_parser(self, filename):
        # .csvfile is a work-around and is used for exporting fees to a new CSV file
        AirBankCZPlugin.csvfile = re.sub(".csv", "", filename) + "-fees.csv"

        # Encoding may be set in Air Bank internet banking (Settings / Applications)
        AirBankCZPlugin.encoding = self.settings.get('charset', 'utf-8')
        f = open(filename, "r", encoding=AirBankCZPlugin.encoding)
        parser = AirBankCZParser(f)
        parser.statement.currency = self.settings.get('currency', 'CZK')
        parser.statement.bank_id = self.settings.get('bank', 'AIRACZPP')
        parser.statement.account_id = self.settings.get('account', '')
        parser.statement.account_type = self.settings.get('account_type', 'CHECKING')
        parser.statement.trntype = "OTHER"
        return parser


class AirBankCZParser(CsvStatementParser):

    # The columns are:
    # 00 Datum provedení
    # 01 Směr platby
    # 02 Typ platby
    # 03 Skupina plateb
    # 04 Měna účtu
    # 05 Částka v měně účtu
    # 06 Poplatek v měně účtu
    # 07 Původní měna platby
    # 08 Původní částka platby
    # 09 Název protistrany
    # 10 Číslo účtu protistrany
    # 11 Název účtu protistrany
    # 12 Variabilní symbol
    # 13 Konstantní symbol
    # 14 Specifický symbol
    # 15 Zdrojová obálka
    # 16 Cílová obálka
    # 17 Poznámka pro mne
    # 18 Zpráva pro příjemce
    # 19 Poznámka k platbě
    # 20 Název karty
    # 21 Číslo karty
    # 22 Držitel karty
    # 23 Obchodní místo
    # 24 Směnný kurz
    # 25 Odesílatel poslal
    # 26 Poplatky jiných bank
    # 27 Datum a čas zadání
    # 28 Datum splatnosti
    # 29 Datum schválení
    # 30 Datum zaúčtování
    # 31 Referenční číslo
    # 32 Způsob zadání
    # 33 Zadal
    # 34 Zaúčtováno
    # 35 Pojmenování příkazu
    # 36 Název, adresa a stát protistrany
    # 37 Název, adresa a stát banky protistrany
    # 38 Typ poplatku
    # 39 Účel platby
    # 40 Zvláštní pokyny k platbě
    # 41 Související platby

    mappings = {"date": 0,
                "memo": 19,
                "payee": 9,
                "amount": 5,
                "check_no": 12,
                "refnum": 31, }

    date_format = "%d/%m/%Y"

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        # Field delimiter may be set in Air Bank internet banking (Settings / Applications)
        return csv.reader(self.fin, delimiter=',', quotechar='"')

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """

        # Ignore the 1st line of CSV
        if self.cur_record == 1:
            # Create a heading line for the -fees.csv file
            with open(AirBankCZPlugin.csvfile, "w", encoding=AirBankCZPlugin.encoding) as output:
                writer = csv.writer(output, lineterminator='\n', delimiter=',', quotechar='"')
                writer.writerow(line)
                output.close()
            # And skip further processing by parser
            return None

        if line[5] == '':
            line[5] = "0"
        if line[6] == '':
            line[6] = "0"

        StatementLine = super(AirBankCZParser, self).parse_record(line)

        # Ignore lines, which do not have posting date yet (typically pmts by debet cards
        # have some delays).
        if not line[30]:
            return None
        else:
            StatementLine.date_user = line[30]
            StatementLine.date_user = datetime.strptime(StatementLine.date_user, self.date_format)

        StatementLine.id = statement.generate_transaction_id(StatementLine)

        # Manually set some of the known transaction types
        if line[2].startswith("Daň z úroku"):
            StatementLine.trntype = "DEBIT"
        elif line[2].startswith("Kreditní úrok"):
            StatementLine.trntype = "INT"
        elif line[2].startswith("Poplatek za "):
            StatementLine.trntype = "FEE"
        elif line[2].startswith("Příchozí platba"):
            StatementLine.trntype = "XFER"
        elif line[2].startswith("Odchozí platba"):
            StatementLine.trntype = "XFER"
        elif line[2].startswith("Výběr hotovosti"):
            StatementLine.trntype = "ATM"
        elif line[2].startswith("Platba kartou"):
            StatementLine.trntype = "POS"
        elif line[2].startswith("Inkaso"):
            StatementLine.trntype = "DIRECTDEBIT"
        elif line[2].startswith("Trvalý"):
            StatementLine.trntype = "REPEATPMT"
        else:
            StatementLine.trntype = "XFER"

        # .payee becomes OFX.NAME which becomes "Description" in GnuCash
        # .memo  becomes OFX.MEMO which becomes "Notes"       in GnuCash
        # When .payee is empty, GnuCash imports .memo to "Description" and keeps "Notes" empty

        # StatementLine.payee = "Název protistrany" + "Číslo účtu protistrany"
        if not line[10] == "":
            StatementLine.payee = StatementLine.payee + "|ÚČ: " + line[10]

        # StatementLine.memo = "Poznámka k platbě" + the payment identifiers
        if not (line[12] == "" or line[12] == " "):
            StatementLine.memo = StatementLine.memo + "|VS: " + line[12]

        if not (line[13] == "" or line[13] == " "):
            StatementLine.memo = StatementLine.memo + "|KS: " + line[13]

        if not (line[14] == "" or line[14] == " "):
            StatementLine.memo = StatementLine.memo + "|SS: " + line[14]

        if not (line[20] == "" or line[20] == " "):
            StatementLine.memo = StatementLine.memo + "|Název karty: " + line[20]

        # Some type of fee is standalone, not related to transaction amount. Add it to amount field.only
        if float(line[6]) != 0 and StatementLine.amount == 0:
            StatementLine.amount = float(line[6])

        # Air Bank may show various fees on the same line  as the underlying transaction
        # For now, we simply create a new CSV file with the fee (and only the fee) moved to line[5].
        # This new file -fees.csv needs to be processed again manually:
        #     $ ofxstatement convert -t raiffeisencz in-fees.csv out-fees.ofx

        # ToDo: instead of exporting the above to CSV, try to add the exportline to
        #       the end of statement (from imported input.csv).
        if float(line[6]) != 0 and StatementLine.amount != 0:
            exportline = line[:]
            exportline[5] = line[6]
            exportline[6] = ''
            exportline[3] = "Poplatek za transakci"
            exportline[19] = "Poplatek: " + exportline[19]

            with open(AirBankCZPlugin.csvfile, "a", encoding=AirBankCZPlugin.encoding) as output:
                writer = csv.writer(output, lineterminator='\n', delimiter=',', quotechar='"')
                writer.writerow(exportline)

        if StatementLine.amount == 0:
            return None

        return StatementLine
